@@ -35,15 +35,16 @@
 #
 from subprocess   import run as subprocess_run, PIPE
 from dataclasses  import dataclass, make_dataclass, field
-from datetime     import datetime
+from datetime     import date, time, datetime
 from pathlib      import Path
 from os           import environ
-from typing       import Union
+from typing       import Union, Any
 
 from flags                      import Flags
 from pyCommonClasses.Version    import Version
 from pyTerminalUI               import ILineTerminal
 
+from pyVersioning.Utils         import SelfDescriptive
 from pyVersioning.AppVeyor      import AppVeyor
 from pyVersioning.CIService     import WorkStation
 from pyVersioning.Configuration import Configuration
@@ -53,26 +54,41 @@ from pyVersioning.Travis        import Travis
 
 
 @dataclass
-class Tool():
+class Tool(SelfDescriptive):
 	name    : str
 	version : Version
 
+	_public = ['name', 'version']
 
-@dataclass
-class Commit():
-	hash : str
-	date : datetime
+
+class Date(date, SelfDescriptive):
+	_public = ['day', 'month', 'year']
+
+
+class Time(time, SelfDescriptive):
+	_public = ['hour', 'minute', 'second']
 
 
 @dataclass
-class Git():
+class Commit(SelfDescriptive):
+	hash: str
+	date: Date
+	time: Time
+
+	_public = ['hash', 'date', 'time']
+
+
+@dataclass
+class Git(SelfDescriptive):
 	commit      : Commit
 	reference   : str = field(init=False)
 	tag         : str = ""
 	branch      : str = ""
 	repository  : str = ""
 
-	def __post_init__(self):
+	_public = ['commit', 'reference', 'tag', 'branch', 'repository']
+
+	def __post_init__(self) -> None:
 		"""Calculate `reference` from `tag` or `branch`."""
 
 		if self.tag != "":
@@ -84,12 +100,14 @@ class Git():
 
 
 @dataclass
-class Project():
-	name    : str
-	variant : str
-	version : Version
+class Project(SelfDescriptive):
+	name:     str
+	variant:  str
+	version:  Version
 
-	def __init__(self, name: str, version: Union[str, Version] = None, variant: str = None):
+	_public = ['name', 'variant', 'version']
+
+	def __init__(self, name: str, version: Union[str, Version] = None, variant: str = None) -> None:
 		"""Assign fields and convert version string to a `Version` object."""
 
 		self.name    = name    if name    is not None else ""
@@ -104,13 +122,15 @@ class Project():
 
 
 @dataclass
-class Compiler():
-	name          : str
-	version       : Version
-	configuration : str
-	options       : str
+class Compiler(SelfDescriptive):
+	name:           str
+	version:        Version
+	configuration:  str
+	options:        str
 
-	def __init__(self, name : str, version : Union[str, Version] = "", configuration : str = "", options : str = ""):
+	_public = ['name', 'version', 'configuration', 'options']
+
+	def __init__(self, name: str, version: Union[str, Version] = "", configuration: str = "", options: str = "") -> None:
 		"""Assign fields and convert version string to a `Version` object."""
 
 		self.name          = name          if name          is not None else ""
@@ -126,9 +146,12 @@ class Compiler():
 
 
 @dataclass
-class Build():
-	date     : datetime
-	compiler : Compiler
+class Build(SelfDescriptive):
+	date:     Date
+	time:     Time
+	compiler: Compiler
+
+	_public = ['date', 'time', 'compiler']
 
 
 class Platforms(Flags):
@@ -143,7 +166,7 @@ class Versioning(ILineTerminal):
 	platform  : int = Platforms.Workstation
 	variables : dict
 
-	def __init__(self, terminal : ILineTerminal):
+	def __init__(self, terminal: ILineTerminal):
 		super().__init__(terminal)
 
 		self.variables = {}
@@ -159,14 +182,14 @@ class Versioning(ILineTerminal):
 		else:
 			self.platform = Platforms.Workstation
 
-	def loadDataFromConfiguration(self, config : Configuration):
+	def loadDataFromConfiguration(self, config: Configuration) -> None:
 		"""Preload versioning information from configuration file."""
 
 		self.variables['project'] = self.getProject(config.project)
 		self.variables['build']   = self.getBuild(config.build)
 		self.variables['version'] = self.getVersion(config.project)
 
-	def collectData(self):
+	def collectData(self) -> None:
 		"""Collect versioning information from environment including CI services (if available)."""
 
 		if self.platform is Platforms.AppVeyor:
@@ -184,24 +207,24 @@ class Versioning(ILineTerminal):
 		else:
 			self.service                = WorkStation()
 
-		self.variables['tool']     = Tool("pyVersioning", Version(0,7,1)),
+		self.variables['tool']     = Tool("pyVersioning", Version(0,7,1))
 		self.variables['git']      = self.getGitInformation()
 		self.variables['env']      = self.getEnvironment()
 		self.variables['platform'] = self.service.getPlatform()
 
 		self.calculateData()
 
-	def calculateData(self):
+	def calculateData(self) -> None:
 		if self.variables['git'].tag != "":
 			pass
 
-	def getVersion(self, config : Configuration.Project) -> Version:
+	def getVersion(self, config: Configuration.Project) -> Version:
 		if config.version is not None:
 			return config.version
 		else:
 			return Version("0.0.0")
 
-	def getGitInformation(self):
+	def getGitInformation(self) -> Git:
 		return Git(
 			commit=self.getLastCommit(),
 			tag=self.getGitTag(),
@@ -209,13 +232,15 @@ class Versioning(ILineTerminal):
 			repository=self.getGitRemoteURL()
 		)
 
-	def getLastCommit(self):
+	def getLastCommit(self) -> Commit:
+		dt = self.getCommitDate()
 		return Commit(
 			hash=self.getGitHash(),
-			date=self.getCommitDate()
+			date=dt.date(),
+			time=dt.time()
 		)
 
-	def getGitHash(self):
+	def getGitHash(self) -> str:
 		if self.platform is not Platforms.Workstation:
 			return self.service.getGitHash()
 
@@ -230,20 +255,22 @@ class Versioning(ILineTerminal):
 			message = completed.stderr.decode('utf-8')
 			self.WriteFatal("Message from '{command}': {message}".format(command=command, message=message))
 
-	def getCommitDate(self):
+	def getCommitDate(self) -> datetime:
 		try:
 			command =   "git show -s --format=%ct HEAD"
 			completed = subprocess_run(command, stdout=PIPE, stderr=PIPE)
 		except:
-			return None
+			raise Exception
+
 		if completed.returncode == 0:
 			ts = int(completed.stdout.decode('utf-8').split("\n")[0])
 			return datetime.fromtimestamp(ts)
 		else:
 			message = completed.stderr.decode('utf-8')
 			self.WriteFatal("Message from '{command}': {message}".format(command=command, message=message))
+			raise Exception
 
-	def getGitLocalBranch(self):
+	def getGitLocalBranch(self) -> str:
 		if self.platform is not Platforms.Workstation:
 			return self.service.getGitBranch()
 
@@ -258,7 +285,7 @@ class Versioning(ILineTerminal):
 			message = completed.stderr.decode('utf-8')
 			self.WriteFatal("Message from '{command}': {message}".format(command=command, message=message))
 
-	def getGitRemoteBranch(self, localBranch : str = None):
+	def getGitRemoteBranch(self, localBranch: str = None) -> str:
 		if localBranch is None:
 			localBranch = self.getGitLocalBranch()
 
@@ -266,14 +293,16 @@ class Versioning(ILineTerminal):
 			command =   "git config branch.{localBranch}.merge".format(localBranch=localBranch)
 			completed = subprocess_run(command, stdout=PIPE, stderr=PIPE)
 		except:
-			return ""
+			raise Exception
+
 		if completed.returncode == 0:
 			return completed.stdout.decode('utf-8').split("\n")[0]
 		else:
 			message = completed.stderr.decode('utf-8')
 			self.WriteFatal("Message from '{command}': {message}".format(command=command, message=message))
+			raise Exception
 
-	def getGitRemote(self, localBranch : str = None):
+	def getGitRemote(self, localBranch: str = None) -> str:
 		if localBranch is None:
 			localBranch = self.getGitLocalBranch()
 
@@ -281,7 +310,8 @@ class Versioning(ILineTerminal):
 			command =   "git config branch.{localBranch}.remote".format(localBranch=localBranch)
 			completed = subprocess_run(command, stdout=PIPE, stderr=PIPE)
 		except:
-			return ""
+			raise Exception
+
 		if completed.returncode == 0:
 			return completed.stdout.decode('utf-8').split("\n")[0]
 		elif completed.returncode == 1:
@@ -290,8 +320,9 @@ class Versioning(ILineTerminal):
 		else:
 			message = completed.stderr.decode('utf-8')
 			self.WriteFatal("Message from '{command}': {message}".format(command=command, message=message))
+			raise Exception
 
-	def getGitTag(self):
+	def getGitTag(self) -> str:
 		if self.platform is not Platforms.Workstation:
 			return self.service.getGitTag()
 
@@ -299,41 +330,47 @@ class Versioning(ILineTerminal):
 			command =   "git tag --points-at HEAD"
 			completed = subprocess_run(command, stdout=PIPE, stderr=PIPE)
 		except:
-			return ""
+			raise Exception
+
 		if completed.returncode == 0:
 			return completed.stdout.decode('utf-8').split("\n")[0]
 		else:
 			message = completed.stderr.decode('utf-8')
 			self.WriteFatal("Message from '{command}': {message}".format(command=command, message=message))
+			raise Exception
 
-	def getGitRemoteURL(self, remote : str = None):
+	def getGitRemoteURL(self, remote: str = None) -> str:
 		if remote is None:
 			remote = self.getGitRemote()
 		try:
 			command =   "git config remote.{remote}.url".format(remote=remote)
 			completed = subprocess_run(command, stdout=PIPE, stderr=PIPE)
 		except:
-			return ""
+			raise Exception
+
 		if completed.returncode == 0:
 			return completed.stdout.decode('utf-8').split("\n")[0]
 		else:
 			message = completed.stderr.decode('utf-8')
 			self.WriteFatal("Message from '{command}': {message}".format(command=command, message=message))
+			raise Exception
 
-	def getProject(self, config : Configuration.Project):
+	def getProject(self, config: Configuration.Project) -> Project:
 		return Project(
 			name=config.name,
 			version=config.version,
 			variant=config.variant
 		)
 
-	def getBuild(self, config : Configuration.Build):
+	def getBuild(self, config: Configuration.Build) -> Build:
+		dt = datetime.now()
 		return Build(
-			date=datetime.now(),
+			date=dt.date(),
+			time=dt.time(),
 			compiler=self.getCompiler(config.compiler)
 		)
 
-	def getCompiler(self, config : Configuration.Build.Compiler):
+	def getCompiler(self, config: Configuration.Build.Compiler) -> Compiler:
 		return Compiler(
 			name=config.name,
 			version=Version(config.version),
@@ -341,7 +378,7 @@ class Versioning(ILineTerminal):
 			options=config.options
 		)
 
-	def getEnvironment(self):
+	def getEnvironment(self) -> Any:
 		env = {}
 		for key, value in environ.items():
 			if not key.isidentifier():
@@ -352,19 +389,26 @@ class Versioning(ILineTerminal):
 			key = key.replace(" ", "_")
 			env[key] = value
 
+		def func(s):
+			for e in env.keys():
+				yield (e, s.__getattribute__(e))
+
 		Environment = make_dataclass(
 			"Environment",
 			[(name, str) for name in env.keys()],
+#			bases=(SelfDescriptive,),
 			namespace={
-				'as_dict': lambda self: env
-			}
+				'as_dict':        lambda self: env,
+				'Keys':           lambda self: env.keys(),
+				'KeyValuePairs':  lambda self: func(self)
+			},
+			repr=True
 		)
 
 		return Environment(**env)
 
-	def writeSourceFile(self, template : Path, filename : Path):
-		with template.open('r') as file:
-			content = template.read_text()
+	def writeSourceFile(self, template: Path, filename: Path) -> None:
+		content = template.read_text()
 
 		# apply variables
 		content = content.format(**self.variables)
